/*
 * Copyright 2016 Resin.io
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

'use strict';

/**
 * @module Etcher.Models.Drives
 */

const angular = require('angular');
const _ = require('lodash');
const MODULE_NAME = 'Etcher.Models.Drives';
const Drives = angular.module(MODULE_NAME, []);

Drives.service('DrivesModel', function() {

  /**
   * @summary List of available drives
   * @type {Object[]}
   * @private
   */
  let availableDrives = [];

  /**
   * @summary Check if there are available drives
   * @function
   * @public
   *
   * @returns {Boolean} whether there are available drives
   *
   * @example
   * if (DrivesModel.hasAvailableDrives()) {
   *   console.log('There are available drives!');
   * }
   */
  this.hasAvailableDrives = function() {
    return !_.isEmpty(availableDrives);
  };

  /**
   * @summary Set a list of drives
   * @function
   * @private
   *
   * @param {Object[]} drives - drives
   *
   * @throws Will throw if no drives
   * @throws Will throw if drives is not an array of objects
   *
   * @example
   * DrivesModel.setDrives([ ... ]);
   */
  this.setDrives = function(drives) {

    if (!drives) {
      throw new Error('Missing drives');
    }

    if (!_.isArray(drives) || !_.every(drives, _.isPlainObject)) {
      throw new Error(`Invalid drives: ${drives}`);
    }

    // Only update if something has changed
    // to avoid unnecessary DOM manipulations
    // angular.equals ignores $$hashKey by default
    if (!angular.equals(availableDrives, drives)) {
      availableDrives = drives;
    }

  };

  /**
   * @summary Get detected drives
   * @function
   * @private
   *
   * @returns {Object[]} drives
   *
   * @example
   * const drives = DrivesModel.getDrives();
   */
  this.getDrives = function() {
    return availableDrives;
  };

});

module.exports = MODULE_NAME;
